@@ -7,7 +7,7 @@ from typing import Optional
 
 
 from azure.storage.blob import ContainerClient, BlobClient, BlobLeaseClient
-from azure.core.exceptions import ResourceNotFoundError, ResourceExistsError
+from azure.core.exceptions import ResourceNotFoundError, ResourceExistsError, HttpResponseError
 
 from ._upath import BlobUpath, LockAcquisitionTimeoutError
 
@@ -107,25 +107,45 @@ class AzureBlobUpath(BlobUpath):
                 t0 = time.perf_counter()
                 while True:
                     try:
-                        t1 = time.perf_counter()
-                        if t1 - t0 > wait:
-                            raise LockAcquisitionTimeoutError(
-                                str(self), t1 - t0)
-                        self._lease_id = self._blob_client.acquire_lease(
-                            lease_duration=60,
-                            timeout=t1 - t0).id
-                        self._t_renew_lease = threading.Thread(
-                            target=self._renew_lease)
-                        self._t_renew_lease.start()
-                        break
-                    except ResourceNotFoundError:
+                        self.write_text(
+                            datetime.utcnow().isoformat(), overwrite=True)
+                    except HttpResponseError as e:
+                        if e.status_code == 412 and e.error_code == 'LeaseIdMissing':
+                            # Blob exists and has a lease on it.
+                            # Proceed to next step---acquire a lease.
+                            time.sleep(0.03)
+                        else:
+                            raise
+
+                    lock_acquired = False
+                    while True:
                         try:
-                            self.write_text(
-                                datetime.utcnow().isoformat(), overwrite=False)
-                        except ResourceExistsError:
-                            # Somehow another worker has just created this blob.
-                            # Continue to wait.
-                            continue
+                            self._lease_id = self._blob_client.acquire_lease(
+                                lease_duration=60,
+                                timeout=1).id
+                            lock_acquired = True
+                            break
+                        except ResourceNotFoundError:
+                            break  # go to the outer looper to write the file.
+                        except HttpResponseError as e:
+                            if e.status_code == 409 and e.error_code == 'LeaseAlreadyPresent':
+                                # Having a lease held by others. Continue to wait.
+                                pass
+                            else:
+                                raise
+
+                        t1 = time.perf_counter()
+                        if t1 - t0 >= wait:
+                            raise LockAcquisitionTimeoutError(self, t1 - t0)
+                        time.sleep(0.03)
+
+                    if lock_acquired:
+                        break
+
+                self._t_renew_lease = threading.Thread(
+                    target=self._renew_lease)
+                self._t_renew_lease.start()
+
             self._lock_count += 1
             try:
                 yield
@@ -137,10 +157,8 @@ class AzureBlobUpath(BlobUpath):
                     self._t_renew_lease_stopped = False
                     self._t_renew_lease = None
                     self.rm()
-                    BlobLeaseClient(self._blob_client,
-                                    lease_id=self._lease_id).release()
-                    # TODO:
-                    # is the order of the two statements above correct?
+                    # BlobLeaseClient(self._blob_client,
+                    #                 lease_id=self._lease_id).release()
                     self._lease_id = None
                     self._lock_count = 0
 
@@ -200,7 +218,7 @@ class AzureBlobUpath(BlobUpath):
             time.sleep(0.012)
             if self._t_renew_lease_stopped:
                 return
-            if time.perf_counter() - t0 >= 57:
+            if time.perf_counter() - t0 >= 13:
                 # Renew ahead of the lease duration 60 seconds.
                 BlobLeaseClient(self._blob_client,
                                 lease_id=self._lease_id).renew()
@@ -214,7 +232,9 @@ class AzureBlobUpath(BlobUpath):
                 if self.is_dir():
                     raise IsADirectoryError(self)
                 raise FileNotFoundError(self)
-            self._blob_client.delete_blob(delete_snapshots='include')
+            self._blob_client.delete_blob(
+                delete_snapshots='include',
+                lease=self._lease_id)
             return 1
 
     def stat(self):
@@ -230,7 +250,16 @@ class AzureBlobUpath(BlobUpath):
 
     def write_bytes(self, data, *, overwrite=False):
         with self._provide_blob_client():
-            super().write_bytes(data, overwrite=overwrite)
+            # if self.is_dir():
+            #     raise IsADirectoryError(self)
+            if self._path == '/':
+                raise IsADirectoryError(self)
+            # p = self.parent
+            # while p._path != '/':
+            #     if p.is_file():
+            #         raise FileExistsError(p)
+            #     p = p.parent
+
             nbytes = len(data)
             try:
                 self._blob_client.upload_blob(
