@@ -17,7 +17,8 @@ class GcpBlobUpath(BlobUpath):
             project=account_info['project_id'],
             credentials=gcp_cred,
         )
-        self._bucket_name = self._client.get_bucket(bucket_name)
+        self._bucket_name = bucket_name
+        self._bucket = self._client.get_bucket(bucket_name)
 
     def __repr__(self) -> str:
         return "{}('{}', bucket_name='{}'".format(
@@ -98,7 +99,18 @@ class GcpBlobUpath(BlobUpath):
         return {}
 
     def write_bytes(self, data, *, overwrite=False):
-        super().write_bytes(data, overwrite=overwrite)
+        # if self.is_dir():
+        #     raise IsADirectoryError(self)
+        if self._path == '/':
+            raise IsADirectoryError(self)
+        # p = self.parent
+        # while p._path != '/':
+        #     if p.is_file():
+        #         raise FileExistsError(p)
+        #     p = p.parent
+
+        nbytes = len(data)
+
         b = self._bucket.blob(self._path.lstrip('/'))
         if b.exists():
             if not overwrite:
@@ -107,3 +119,4 @@ class GcpBlobUpath(BlobUpath):
         if isinstance(data, BufferedReader):
             data = data.read()
         b.upload_from_string(data)
+        return nbytes
